
# Worksheet 1: Assembly Programming with Docker Integration

This project is part of the **Worksheet 1** task, focusing on assembly programming and integration with Docker for build and run processes. Due to connectivity issues with `csctcloud`, Docker was used as a workaround to set up a Linux environment directly from Visual Studio Code. This approach allowed for seamless development, testing, and debugging of assembly programs.

---

## Project Overview

This repository contains assembly language programs and supporting files to demonstrate basic programming concepts in assembly language. The workflow involves:
- Writing assembly code.
- Integrating it with C for execution.
- Automating builds with a `Makefile`.
- Running the tasks in a containerized Linux environment using Docker.

---

## Docker Integration

### Why Docker?
`csctcloud` environment was unavailable due to connectivity issues. To continue working efficiently, Docker was used to:
- Set up a 32-bit Linux environment required for assembly programming.
- Provide a consistent environment directly accessible through Visual Studio Code for editing, building, and running tasks.

### Dockerfile Configuration
The Dockerfile is configured to create an environment with all necessary tools and dependencies for assembly programming. The following packages are installed during the build process:

1. **`nasm`**: An assembler required to compile `.asm` files into object files.
2. **`gcc`**: Used to compile C programs and link them with assembly object files.
3. **`gcc-multilib`**: Provides support for compiling and linking 32-bit binaries on a 64-bit system.
4. **`libc6-dev`**: Development libraries for the GNU C Library (glibc), providing headers and support for 32-bit compilation.

### Docker Build Process
During the Docker image build, these dependencies are installed with the following commands:
```bash
apt update
apt install -y nasm gcc
apt install -y gcc-multilib
apt install -y libc6-dev
```
---

### Using Docker
1. **Build the Docker Image**:
   ```bash
   docker build -t asm_project .
   ```
2. **Run the Docker Container**:
   ```bash
   docker run -it --rm -v $(pwd):/workspace -w /workspace asm_project
   ```
3. **Inside the Container**:
   - Compile all tasks:
     ```bash
     make
     ```
   - Run each task:
     - Task 1:
       ```bash
       ./src/task1
       ```
     - Task 2:
       ```bash
       ./src/task2
       ```

   - Clean up object files and executables:
     ```bash
     make clean
     ```
---


# Tasks: Detailed Code Explanations

## Task 1: Adding Two Integers
- **File**: `task1.asm`
- **Description**: This task adds two integers stored in global memory and prints the result.

### Code Explanation:

#### Defining Variables:
```assembly
SECTION .data
num1 dd 5           ; First integer
num2 dd 10          ; Second integer
result dd 0         ; Memory location to store the result
```
- Defines two integers, `num1` and `num2`, with initial values.
- Allocates a memory location for `result` to store the sum.

#### Main Logic:
```assembly
mov eax, [num1]     ; Load num1 into register eax
add eax, [num2]     ; Add num2 to eax
mov [result], eax   ; Store the result in memory
```
- Loads `num1` into the `eax` register.
- Adds `num2` to the value in `eax`.
- Stores the result back into the `result` memory location.

#### Printing the Result:
```assembly
push dword [result]
call _print_int
```
- Pushes the value of `result` onto the stack.
- Calls the `_print_int` function to display the result.

#### How to Compile and Run:
1. Compile the assembly file:
   ```bash
   nasm -f elf32 src/task1.asm -o src/task1.o
   ```
2. Link the compiled task1.o, driver.o, and print_int.o files using gcc to create an executable file:
   ```bash
   gcc -m32 src/task1.o src/driver.o src/print_int.o -o src/task1
   ```
3. Run the executable:
   ```bash
   ./src/task1
   ```
# Task 1 Output: Assembly Programming with Docker Integration

## Screenshot and Explanation

**Description**: This screenshot demonstrates the result of Task 1, where two integers (`10` and `20`) are added using assembly code. The result (`30`) is displayed in the terminal.

![Task 1 Output](images/photo1.png)

---

## Task 2: Loops and Conditionals
- **File**: `task2.asm`
- **Description**: Implements a loop to sum an array of integers and validates user input.

### Code Explanation:

#### Defining Variables:
```assembly
SECTION .data
array dd 1, 2, 3, 4, 5  ; Array of integers
length equ 5             ; Length of the array
sum dd 0                 ; To store the sum
```
- Defines an array of integers `array`.
- Uses `length` to store the size of the array.
- Allocates a memory location for `sum` to store the total.

#### Summing the Array:
```assembly
mov ecx, length      ; Set loop counter
xor eax, eax         ; Clear accumulator
xor edi, edi         ; Array index
sum_loop:
    add eax, [array + edi*4] ; Add array element to eax
    inc edi                  ; Increment index
    loop sum_loop            ; Decrement ecx and repeat
mov [sum], eax       ; Store the sum in memory
```
- Initializes the loop counter with the array length.
- Iterates through the array, adding each element to `eax`.
- Stores the total in `sum`.

#### Validating Input:
```assembly
cmp eax, 50
jl invalid_input
cmp eax, 100
jg invalid_input
```
- Compares the total with a valid range (50 to 100).
- Jumps to `invalid_input` if the total is out of range.

#### Printing the Sum:
```assembly
push dword [sum]
call _print_int
```
- Pushes the value of `sum` onto the stack.
- Calls the `_print_int` function to display the sum.

#### How to Compile and Run:
1. Compile the assembly file:
   ```bash
   nasm -f elf32 src/task2.asm -o src/task2.o
   ```
2. Link the compiled task2.o, driver.o, and print_int.o files using gcc to create an executable file:
   ```bash
   gcc -m32 src/task2.o src/driver.o src/print_int.o -o src/task2

   ```
3. Run the executable:
   ```bash
   ./src/task2
   ```


## Screenshot of Task 2
The following screenshot demonstrates the successful execution of Task 2. It showcases the program's ability to validate user input and sum an array of integers within a specified range.

![Task 2 Output](images/photo2.png)

## Explanation
In Task 2, the program performs two main functions:
1. Validates user input to ensure the entered number is within a specified range (50–100).
2. Sums integers from an array and displays the result.

---

### Task 3: Makefile
- **File**: `Makefile`
- **Description**: This project uses a `Makefile` to streamline the process of building and cleaning executables for Task 1 and Task 2. The `Makefile` automates assembling, compiling, and linking multiple files while maintaining dependencies between them.


#### Defining Variables:
1. task1:
The task1 target compiles and links the assembly and C files required for Task 1, creating the task1 executable.
   ```bash
   task1: src/task1.o src/driver.o src/asm_io.o
	   gcc -m32 -o src/task1 src/task1.o src/driver.o src/asm_io.o

   ```
Dependencies:

-src/task1.o: Compiled from src/task1.asm.
-src/driver.o: Compiled from src/driver.c.
-src/asm_io.o: Compiled from src/asm_io.asm.

2. task2: The task2 target compiles and links the assembly and C files required for Task 2, creating the task2 executable.
   ```assembly
   task2: src/task2.o src/driver.o src/asm_io.o src/print_int.o
	   gcc -m32 -o src/task2 src/task2.o src/driver.o src/asm_io.o src/print_int.o

   ```
Dependencies:

src/task2.o: Compiled from src/task2.asm.
src/driver.o: Compiled from src/driver.c.
src/asm_io.o: Compiled from src/asm_io.asm.
src/print_int.o: Compiled from src/print_int.c.

3.Object File Targets
1.task1 
   ```assembly
   src/task1.o: src/task1.asm
	   nasm -f elf src/task1.asm -o src/task1.o

   ```
2.task2
   ```bash
   src/task2.o: src/task2.asm
	   nasm -f elf src/task2.asm -o src/task2.o
   ```
3.driver 
```bash
   src/driver.o: src/driver.c
	   gcc -m32 -c src/driver.c -o src/driver.o
   ```
4.Assembly I/O 
  ```bash
   src/asm_io.o: src/asm_io.asm
	   nasm -f elf src/asm_io.asm -o src/asm_io.o
   ```
5.Print Integer 
   ```bash
   src/print_int.o: src/print_int.c
	   gcc -m32 -c src/print_int.c -o src/print_int.o

   ```   
5.clean 
 ```bash
   clean:
	   rm -f src/*.o src/task1 src/task2

   ```  

# Task 3 Output

## Screenshot of Task 3

The following Makefile automates the build process for Task 1 and Task 2, ensuring efficient compilation, linking, and cleanup of source files and dependencies within the project.
![Makefile make](images/photo3.png)
![Makefile make clean](images/photo4.png)

## Explanation
The Makefile automates the following operations for Task 1 and Task 2:

1. Assembles the .asm files (task1.asm and task2.asm) into object files.
2. Compiles the .c files (driver.c and print_int.c) into object files.
3. Links all necessary object files to create the task1 and task2 executables.
4. Includes a clean target to remove all generated files, keeping the project directory tidy.

---
## Project Structure

```
worksheet_1/
├── README.md         # Project documentation
├── Makefile          # Automates the build process
├── Dockerfile        # Configures the Linux environment
├── images/           # Directory containing screenshots for documentation
│   ├── photo1.png    # Task 1 screenshot
│   ├── photo2.png    # Task 2 screenshot
│   ├── photo3.png    # Task 3 screenshot
│   ├── photo4.png    # Additional screenshot
├── src/              # Directory containing all source files, object files, and executables
│   ├── asm_io.asm    # Provided Assembly IO code
│   ├── asm_io.inc    # Provided IO include file
│   ├── driver.c      # Main C program calling assembly functions
│   ├── print_int.c   # Helper C program for printing integers
│   ├── task1.asm     # Assembly code for Task 1
│   ├── task2.asm     # Assembly code for Task 2
│   ├── asm_io.o      # Object file for asm_io.asm
│   ├── driver.o      # Object file for driver.c
│   ├── print_int.o   # Object file for print_int.c
│   ├── task1.o       # Object file for task1.asm
│   ├── task2.o       # Object file for task2.asm
│   ├── task1         # Compiled Task 1 executable
│   ├── task2         # Compiled Task 2 executable

```


## Authors

- Hazal Veziroglu

---

## License

This project is licensed under the MIT License. See the `LICENSE` file for details.

---

## Acknowledgments

- UWE Course Material on Assembly Programming.
- Online assembly programming resources and communities.
